using GraphicsManager.Interfaces;
using GraphicsManager.Objects;
using Luski.net;
using Luski.net.Enums;
using OpenTK.Mathematics;
using OpenTK.Windowing.Common;
using OpenTK.Windowing.GraphicsLibraryFramework;

namespace Luski.GUI.StartPage.UI;

public class CreateAccount : UserControl
{
    private RoundedButton button;
    private Textbox Password, Email, Username;
    private Rectangle rec;
    private string pfp = null;
    
    public event Func<Task> ChangeToApp;
    
    public CreateAccount()
    {
        Size = new(481, 838);
        Controls.Add(new Rectangle(Globals.LuskiTexture) { Location = new(103,8), Size = new(276, 289)});
        Controls.Add(new Label()   { Scale = 1.4f, Location = new(173,305), Text = "Luski", Color = new(243, 119, 53, 255) });
        Controls.Add(new Label()   { Location = new(41,395), Text = "Email"});
        Controls.Add(Email = new Textbox() { Location = new(41,431), Size = new(401,41), InsideColor = new(28,28,28,255), BorderColor = Color4.DarkCyan });
        Controls.Add(new Label()   { Location = new(41,490), Text = "Password" });
        Controls.Add(Password = new Textbox() { PasswordChar = '●', Location = new(41,531), Size = new(401, 41), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(new Label()   { Location = new(41,580), Text = "Display Name" });
        Controls.Add(Username = new Textbox() { Location = new(41,616), Size = new(300, 41), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(button = new() { Text = "Create Account", Location = new(41, 700), Size = new(401, 71), InsideColor = new(28, 28, 28, 255), BorderColor = Color4.DarkCyan });
        Controls.Add(rec = new Rectangle(){ Location = new(350, 585), Size = new(100), BackgroundColor = Color4.Red});
        Password.KeyPress += PasswordOnKeyPress;
        Email.KeyPress += EmailOnKeyPress;
        button.Clicked += ButtonOnClicked;
        Username.KeyPress += UsernameOnKeyPress;
        rec.FilesDroped += RecOnFilesDroped;
    }

    private Task UsernameOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) { Password.Focus(); return Task.CompletedTask; } 
        if (arg.Key == Keys.Enter || arg.Key == Keys.KeyPadEnter) ButtonOnClicked(Username);
        return Task.CompletedTask;
    }

    private Task RecOnFilesDroped(string[] arg)
    {
        if (!arg[0].ToLower().EndsWith("png")) return Task.CompletedTask;
        Controls.Remove(rec);
        pfp = arg[0];
        rec = new(new(File.ReadAllBytes(arg[0]))){ Location = new(350, 585), Size = new(100) };
        Controls.Add(rec);
        Window!.DrawFrame();
        return Task.CompletedTask;
    }

    private bool tab;
    private Task EmailOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab){ Password.Focus(); tab = true;}
        return Task.CompletedTask;
    }

    private Task PasswordOnKeyPress(KeyboardKeyEventArgs arg)
    {
        if (tab)
        {
            tab = false;
            return Task.CompletedTask;
        };
        if (arg.Key != Keys.Enter && arg.Key != Keys.KeyPadEnter && arg.Key != Keys.Tab) return Task.CompletedTask;
        if (arg.Key == Keys.Tab && arg.Shift) { Email.Focus(); return Task.CompletedTask; }
        if (arg.Key == Keys.Tab ){ Username.Focus(); return Task.CompletedTask; }
        return Task.CompletedTask;
    }

    private Task ButtonOnClicked(IRenderObject arg)
    {
        try
        {
            string[] arr = new string[]
            {
                Email.Text,
                Password.Text,
                Username.Text,
                pfp
            };
            if (arr.Any(s => string.IsNullOrEmpty(s))) return Task.CompletedTask;
            Globals.Luski = Server.CreateAccount(Email.Text, Password.Text, Username.Text, pfp, Globals.Branch);
            ChangeToApp.Invoke();
        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }
        return Task.CompletedTask;
    }
}